@@ -662,14 +662,13 @@ _tpl_log_manager_get_chats (TplLogManager *manager,
   for (l = priv->readable_stores; l != NULL; l = g_list_next (l))
     {
       TplLogStore *store = TPL_LOG_STORE (l->data);
-      GList *in;
+      GList *in, *j;
 
+      in = _tpl_log_store_get_chats (store, account);
       /* merge the lists avoiding duplicates */
-      for (in = _tpl_log_store_get_chats (store, account);
-          in != NULL;
-          in = g_list_next (in))
+      for (j = in; j != NULL; j = g_list_next (j))
         {
-          TplLogSearchHit *hit = in->data;
+          TplLogSearchHit *hit = j->data;
 
           if (g_list_find_custom (out, hit,
                 (GCompareFunc) _tpl_log_manager_search_hit_compare) == NULL)
@@ -828,12 +827,12 @@ tpl_log_manager_chat_info_new (void)
 static void
 tpl_log_manager_chat_info_free (TplLogManagerChatInfo *data)
 {
-  if (data->account != NULL)
-    g_object_unref (data->account);
-  if (data->chat_id != NULL)
-    g_free (data->chat_id);
-  if (data->date != NULL)
-    g_date_free (data->date);
+  tp_clear_object (&data->account);
+  tp_clear_object (&data->logentry);
+
+  tp_clear_pointer (&data->chat_id, g_free);
+  tp_clear_pointer (&data->date, g_date_free);
+  tp_clear_pointer (&data->search_text, g_date_free);
   g_slice_free (TplLogManagerChatInfo, data);
 }
 
@@ -925,6 +924,15 @@ _tpl_log_manager_add_message_async (TplLogManager *manager,
 
   g_simple_async_result_run_in_thread (simple, _add_message_async_thread, 0,
       NULL);
+
+  g_object_unref (simple);
+}
+
+/* There is no g_date_copy() */
+static GDate *
+copy_date (const GDate *date)
+{
+  return g_date_new_julian (g_date_get_julian (date));
 }
 
 /**
@@ -955,7 +963,17 @@ tpl_log_manager_get_dates_finish (TplLogManager *self,
     return FALSE;
 
   if (dates != NULL)
-    *dates = g_simple_async_result_get_op_res_gpointer (simple);
+    {
+      GList *list, *l;
+
+      *dates = NULL;
+      list = g_simple_async_result_get_op_res_gpointer (simple);
+
+      for (l = list; l != NULL; l = g_list_next (l))
+        *dates = g_list_prepend (*dates, copy_date (l->data));
+
+      *dates = g_list_reverse (*dates);
+    }
 
   return TRUE;
 }
@@ -963,10 +981,9 @@ tpl_log_manager_get_dates_finish (TplLogManager *self,
 static void
 _get_dates_async_result_free (gpointer data)
 {
-  GList *lst = data; /* list of (char *) */
-  g_return_if_fail (data != NULL);
+  GList *lst = data; /* list of (GDate *) */
 
-  g_list_foreach (lst, (GFunc) g_free, NULL);
+  g_list_foreach (lst, (GFunc) g_date_free, NULL);
   g_list_free (lst);
 }
 
@@ -1041,6 +1058,8 @@ tpl_log_manager_get_dates_async (TplLogManager *manager,
 
   g_simple_async_result_run_in_thread (simple, _get_dates_async_thread, 0,
       NULL);
+
+  g_object_unref (simple);
 }
 
 /**
@@ -1071,7 +1090,12 @@ tpl_log_manager_get_messages_for_date_finish (TplLogManager *self,
     return FALSE;
 
   if (messages != NULL)
-    *messages = g_simple_async_result_get_op_res_gpointer (simple);
+    {
+      *messages = g_list_copy (g_simple_async_result_get_op_res_gpointer (
+            simple));
+
+      g_list_foreach (*messages, (GFunc) g_object_ref, NULL);
+    }
 
   return TRUE;
 }
@@ -1081,7 +1105,6 @@ static void
 _get_messages_for_date_async_result_free (gpointer data)
 {
   GList *lst = data; /* list of TPL_ENTRY */
-  g_return_if_fail (data != NULL);
 
   g_list_foreach (lst, (GFunc) g_object_unref, NULL);
   g_list_free (lst);
@@ -1143,8 +1166,7 @@ tpl_log_manager_get_messages_for_date_async (TplLogManager *manager,
   chat_info->account = g_object_ref (account);
   chat_info->chat_id = g_strdup (chat_id);
   chat_info->is_chatroom = is_chatroom;
-  /* There is no g_date_copy() */
-  chat_info->date = g_date_new_julian (g_date_get_julian (date));
+  chat_info->date = copy_date (date);
 
   async_data->manager = g_object_ref (manager);
   async_data->request = chat_info;
@@ -1159,6 +1181,8 @@ tpl_log_manager_get_messages_for_date_async (TplLogManager *manager,
 
   g_simple_async_result_run_in_thread (simple,
       _get_messages_for_date_async_thread, 0, NULL);
+
+  g_object_unref (simple);
 }
 
 /**
@@ -1189,7 +1213,12 @@ tpl_log_manager_get_filtered_messages_finish (TplLogManager *self,
     return FALSE;
 
   if (messages != NULL)
-    *messages = g_simple_async_result_get_op_res_gpointer (simple);
+    {
+      *messages = g_list_copy (g_simple_async_result_get_op_res_gpointer (
+            simple));
+
+      g_list_foreach (*messages, (GFunc) g_object_ref, NULL);
+    }
 
   return TRUE;
 }
@@ -1199,7 +1228,6 @@ static void
 _get_filtered_messages_async_result_free (gpointer data)
 {
   GList *lst = data; /* list of TPL_ENTRY */
-  g_return_if_fail (data != NULL);
 
   g_list_foreach (lst, (GFunc) g_object_unref, NULL);
   g_list_free (lst);
@@ -1279,6 +1307,23 @@ tpl_log_manager_get_filtered_messages_async (TplLogManager *manager,
 
   g_simple_async_result_run_in_thread (simple,
       _get_filtered_messages_async_thread, 0, NULL);
+
+  g_object_unref (simple);
+}
+
+static GList *
+copy_search_hit_list (GList *list)
+{
+  GList *result = NULL;
+  GList *l;
+
+  for (l = list; l != NULL; l = g_list_next (l))
+    {
+      result = g_list_prepend (result,
+          _tpl_log_manager_search_hit_copy (l->data));
+    }
+
+  return g_list_reverse (result);
 }
 
 /**
@@ -1309,22 +1354,16 @@ tpl_log_manager_get_chats_finish (TplLogManager *self,
     return FALSE;
 
   if (chats != NULL)
-    *chats = g_simple_async_result_get_op_res_gpointer (simple);
+    {
+      GList *list;
+
+      list = g_simple_async_result_get_op_res_gpointer (simple);
+      *chats = copy_search_hit_list (list);
+    }
 
   return TRUE;
 }
 
-static void
-_get_chats_async_result_free (gpointer data)
-{
-  GList *lst = data; /* list of (gchar *) */
-  g_return_if_fail (data != NULL);
-
-  g_list_foreach (lst, (GFunc) g_free, NULL);
-  g_list_free (lst);
-}
-
-
 static void
 _get_chats_async_thread (GSimpleAsyncResult *simple,
     GObject *object,
@@ -1340,7 +1379,7 @@ _get_chats_async_thread (GSimpleAsyncResult *simple,
   lst = _tpl_log_manager_get_chats (async_data->manager, chat_info->account);
 
   g_simple_async_result_set_op_res_gpointer (simple, lst,
-      _get_chats_async_result_free);
+      (GDestroyNotify) tpl_log_manager_search_free);
 }
 
 /**
@@ -1380,6 +1419,8 @@ tpl_log_manager_get_chats_async (TplLogManager *self,
 
   g_simple_async_result_run_in_thread (simple, _get_chats_async_thread, 0,
       NULL);
+
+  g_object_unref (simple);
 }
 
 gboolean
@@ -1403,7 +1444,12 @@ _tpl_log_manager_search_in_identifier_chats_new_finish (TplLogManager *self,
     return FALSE;
 
   if (chats != NULL)
-    *chats = g_simple_async_result_get_op_res_gpointer (simple);
+    {
+      GList *list;
+
+      list = g_simple_async_result_get_op_res_gpointer (simple);
+      *chats = copy_search_hit_list (list);
+    }
 
   return TRUE;
 }
@@ -1412,7 +1458,6 @@ static void
 _search_in_identifier_chats_new_async_result_free (gpointer data)
 {
   GList *lst = data; /* list of TplSearchHit */
-  g_return_if_fail (data != NULL);
 
   g_list_foreach (lst, (GFunc) _tpl_log_manager_search_hit_free, NULL);
   g_list_free (lst);
@@ -1471,6 +1516,8 @@ _tpl_log_manager_search_in_identifier_chats_new_async (TplLogManager *manager,
 
   g_simple_async_result_run_in_thread (simple,
       _search_in_identifier_chats_new_async_thread, 0, NULL);
+
+  g_object_unref (simple);
 }
 
 /**
@@ -1501,7 +1548,12 @@ tpl_log_manager_search_finish (TplLogManager *self,
     return FALSE;
 
   if (chats != NULL)
-    *chats = g_simple_async_result_get_op_res_gpointer (simple);
+    {
+      GList *list;
+
+      list = g_simple_async_result_get_op_res_gpointer (simple);
+      *chats = copy_search_hit_list (list);
+    }
 
   return TRUE;
 }
@@ -1572,6 +1624,8 @@ tpl_log_manager_search_async (TplLogManager *manager,
 
   g_simple_async_result_run_in_thread (simple, _search_async_thread, 0,
       NULL);
+
+  g_object_unref (simple);
 }
 
 /**
@@ -1594,3 +1648,10 @@ tpl_log_manager_errors_quark (void)
 
   return (GQuark) quark;
 }
+
+TplLogSearchHit *
+_tpl_log_manager_search_hit_copy (TplLogSearchHit *hit)
+{
+  return _tpl_log_manager_search_hit_new (hit->account, hit->chat_id,
+      hit->is_chatroom, hit->filename, hit->date);
+}
