@@ -867,6 +867,8 @@ log_store_xml_get_messages_for_file (TplLogStoreXml *self,
 
           instead_of_channel_path = g_strconcat (
               tp_proxy_get_object_path (account), sender_id, NULL);
+
+          xmlFree (log_id);
           log_id = _tpl_create_message_token (instead_of_channel_path, t,
               pending_id);
 
@@ -1150,13 +1152,6 @@ log_store_xml_get_chats (TplLogStore *store,
   hits = log_store_xml_get_chats_for_dir (self, dir, FALSE, account);
   g_free (dir);
 
-  for (guint i = 0; i < g_list_length (hits); ++i)
-    {
-      TplLogSearchHit *hit;
-      hit = g_list_nth_data (hits, i);
-    }
-
-
   return hits;
 }
 
